// Datei: LCD.C
//
// Ansteuerung der LCD-Anzeige
// im 4 Bit Daten Mode
// Seriell ber 4094
//
// Holger Klabunde
// 30.04.2001
// Compiler CC5x

#include "prozess.h"      /* Der verwendete Prozessor */

#include "bargraph.h" //Pindefinitionen
#include "lcd.h"
#include "protos.h"

unsigned char lcdport; //Register zum merken letzter Zustand 4094

//Schreibt die Symbole fr die Balken in die LCD-Anzeige
void CGInit()
{
 unsigned char i;

 LCDRS_OFF(); //Umschalten auf LCD Kontrollregister
 LCDWriteByte(0b01000000); //CG-RAM Adresse 0 setzen

 LCDRS_ON(); //Umschalten auf Datenregister

 for(i=0; i<8; i++) LCDWriteByte(0x10); //Ein senkrechter Strich
 for(i=0; i<8; i++) LCDWriteByte(0x18); //Zwei senkrechte Striche
 for(i=0; i<8; i++) LCDWriteByte(0x1C); //drei senkrechte Striche
 for(i=0; i<8; i++) LCDWriteByte(0x1E); //vier senkrechte Striche
 for(i=0; i<8; i++) LCDWriteByte(0x1F); //fnf senkrechte Striche

 LCDRS_OFF(); //Umschalten auf LCD Kontrollregister
 LCDWriteByte(0b10000000); //DD-RAM Adresse 0 setzen zum beschreiben LCD
 LCDRS_ON(); //Umschalten auf Datenregister
}

//*******************************************
//Schreibt Byte "lcdport" seriell in den 4094
//Reihenfolge D7..D0
//*******************************************
void ShiftOut()
{
 unsigned char mask;

 mask = 0x80;
 while(mask) //LCD Anzeige
  {
   if(lcdport & mask) { DATOUT_ON(); }  //1 Bit rausschieben
   else             { DATOUT_OFF(); } //0 Bit rausschieben

   CLK_ON();
   NOP;               //1
   CLK_OFF(); //Clockimpuls
   mask >>= 1;
  }

 STROUT_ON();
 NOP;               //1
 STROUT_OFF(); //Bytes bernehmen
}

//*******************************************
// Schreibt ein Datenbyte in die LCD-Anzeige
// Schreibt das Byte in zwei 4 Bit Nibbles
// Angepasst an serielles Schreiben ber 4094
//*******************************************
void LCDWriteByte(unsigned char data)
{
 unsigned char buff;
 
 buff = data >> 4;   //High Nibble des Bytes
 lcdport = lcdport & 0xF0;	//Hier D0-D3 ersma lschen zum
 lcdport |= buff;		//reinodern vom High Nibble

 LCDCS_ON();	      //High Nibble schreiben
 LCDCS_OFF();	

 buff = data & 0x0F;        //Low Nibble des Bytes
 lcdport = lcdport & 0xF0;	//Hier D0-D3 ersma lschen zum
 lcdport |= buff;		//reinodern vom Low Nibble

 LCDCS_ON();	      //Low Nibble schreiben
 LCDCS_OFF();	
}

//**************************************************
//Schreibt einen Text in die LCD-Anzeige
//**************************************************
void LCDWrite(const char *__putstr)
{
 char __ps;
 
 __ps = *__putstr;

  while(__ps>0)
   {
    __putstr++;
    if (__ps== 0) break;
    LCDWriteByte(__ps);
    __ps = *__putstr;
   }
}

//****************************************************
// Springt auf Zeile x und Position y der LCD-Anzeige 
// Gltig fr 2 Zeilen
//****************************************************
void LCDPos(unsigned char zeile,unsigned char position)
{
 unsigned char buff;

 LCDRS_OFF(); //Umschalten auf Kontrollregister

// 0x80 wegen Display Adress Set.
// Erste Position ist bei mir 1, nicht 0.
// Offset's
// Zeile 1 -> 0x00+0x80-0x01=0x7F
// Zeile 2 -> 0x40+0x80-0x01=0xBF 

 switch(zeile)
  {
   case 1: buff=0x7F+position; break;
   case 2: buff=0xBF+position; break;
   default : break;
  }

 LCDWriteByte(buff); 
 LCDRS_ON(); //Umschalten auf Datenregister
}

//*****************************************
// Lscht die LCD Anzeige
// Cursor steht dann auf Position 1,Zeile 1
// Gilt auch fr mehrzeilige Anzeigen
//*****************************************
void LCDCls()
{
 LCDRS_OFF(); //Controlregister
 LCDWriteByte(0x01); //CLS
 LCDRS_ON(); //Umschalten auf Datenregister
 Wait();
}

//*********************************************
//Stellt die Funktionsweise der LCD Anzeige ein
//*********************************************
void LCDInit()
{
 unsigned char i;

 STROUT_OFF(); //4094 Grundeinstellung
 NOP;               //1
 CLK_OFF();
 
 LCDRS_OFF();  //Control Register
 LCDCS_OFF();  //LCD-Anzeige anwhlen

//Software Reset laut Hitachi Datenblatt
 Delay1ms(255); //Mglichen Power On Reset abwarten
 for(i=0; i<3; i++)
  {
   lcdport = lcdport & 0xF0;//D0-D3 ersma lschen
   lcdport |= 0x03;	//Wert reinodern
   LCDCS_ON();	//High Nibble schreiben
   LCDCS_OFF(); //LCD-Anzeige abwhlen
   Delay1ms(100); //Ein bichen warten auf die Anzeige
  }
//Ende Software Reset

//Der erste Schreibzugriff nach dem Reset ist ein 8 Bit
//Zugriff!! Also nur einmal schreiben, nicht Low-High Nibble.

//Um die 4Bit Operation einzuschalten
 lcdport = lcdport & 0xF0;//D0-D3 ersma lschen
 lcdport |= 0x02;	//4 Bit Operation
 LCDCS_ON();	//High Nibble schreiben
 LCDCS_OFF();	//High Nibble schreiben
 Wait();

//Nach einfgen von Wait() = 5ms
//kommt die LCD Anzeige nach dem Einschalten immer.
//Ohne Wait's zeigte sie oft gar nix !
 LCDWriteByte(0x28);    //4 Bit Operation,2 Line,5x7 Font
 Wait();
 LCDWriteByte(0x14); 	//Cursor Move,Right Shift
 Wait();

// LCDWriteByte(0x0F); 	//Display on, Cursor on ,Cursor blink
// LCDWriteByte(0x0E); 	//Display on, Cursor on ,Cursor no blink
 LCDWriteByte(0x0C); 	//Display on, Cursor off ,Cursor no blink
 Wait();

 LCDWriteByte(0x06); 	//Increment, Display Freeze
 Wait();
 LCDWriteByte(0x02); 	//Cursor Home
 Wait();
 LCDRS_ON();		//LCD-Datenregister
}

